# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.BooleanExt.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import six


# pragma mark - Define
# --------------------------------------------------------------------------------
TRUE_STRINGS = ('1', 't', 'true', 'on', 'y', 'yes')
FALSE_STRINGS = ('0', 'f', 'false', 'off', 'n', 'no')


def bool_from_string(subject, strict=False, default=False):
    """
    >>> 根据string判断布尔型
    :param {} subject:
    :param {Boolean} strict:
    :param {Boolean} default:
    """
    """Interpret a string as a boolean.

    A case-insensitive match is performed such that strings matching 't',
    'true', 'on', 'y', 'yes', or '1' are considered True and, when
    `strict=False`, anything else returns the value specified by 'default'.

    Useful for JSON-decoded stuff and config file parsing.

    If `strict=True`, unrecognized values, including None, will raise a
    ValueError which is useful when parsing values passed in from an API call.
    Strings yielding False are 'f', 'false', 'off', 'n', 'no', or '0'.
    """
    if not isinstance(subject, six.string_types):
        subject = six.text_type(subject)

    lowered = subject.strip().lower()

    if lowered in TRUE_STRINGS:
        return True
    elif lowered in FALSE_STRINGS:
        return False
    elif strict:
        acceptable = ', '.join(
            "'%s'" % s for s in sorted(TRUE_STRINGS + FALSE_STRINGS))
        msg = ("Unrecognized value '%(val)s', acceptable values are: %(acceptable)s") % {'val': subject, 'acceptable': acceptable}
        raise ValueError(msg)
    else:
        return default



# -*- coding: utf-8 -*-
"""
jf-ext.EncryptExt.py
~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import os
import uuid
import hashlib


def generate_md5(source):
    """
    >>> 加密: 生成Md5值
    :param {String} source: 源字符串
    :return {String}: MD5值
    """
    md5 = hashlib.md5()
    md5.update(source.encode("utf-8"))
    return md5.hexdigest()


def check_md5(source, md5_val):
    """
    >>> 加密: MD5值校验
    :param {String} source: 源字符串
    :param {String} md5_val: MD5值
    :return {Boolean}: MD5值是否匹配
    """
    new_md5_val = generate_md5(source)
    if new_md5_val == md5_val:
        return True
    else:
        return False


def generate_token():
    """
    >>> 加密: 随机生成TOKEN
    :return {String}: MD5 TOKEN
    """
    return hashlib.sha1(os.urandom(24)).hexdigest()


def gen_id():
    """
    >>> 随机成32位字符串
    """
    return uuid.uuid4().hex


def gen_random_id_by_length(length):
    """
    >>> 随机生成定长字符串
    :param {Integer} len: 字符串长度
    """
    return hashlib.sha1(os.urandom(24)).hexdigest()[:length]



# -*- coding: utf-8 -*-
"""
jf-ext.IpExtend.py
~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import validators
import geoip2.database


def ip_get_country(ip):
    """
    >>> 获取ip地址所在国家
    @params {String} ip:
    @returns: {String} ip所属国家
    """
    try:
        # 验证 IP 有效性
        if not validators.ip_address.ipv4(ip):
            return '未知'
        reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-Country.mmdb')
        ip_obj = reader.country(ip)
        return ip_obj.country.names['zh-CN']
    except Exception:
        return '未知'


def ip_get_city(ip):
    """
    >>> 获取ip地址所在城市
    @params ip:
    :type ip: String
    @returns: {String} ip所属城市
    """
    try:
        # 验证 IP 有效性
        if not validators.ip_address.ipv4(ip):
            return '未知'
        reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-City.mmdb')
        ip_obj = reader.city(ip)
        ip_names = ip_obj.city.names
        if ip_names.get('cn', None):
            return ip_names['cn']
        return ip_names['en']
    except Exception:
        return '未知'


def ip_get_time_zone(ip):
    """
    >>> 获取ip地址所在时区
    @params {String} ip:
    @returns: {String} ip所属时区
    """
    try:
        # 验证 IP 有效性
        if not validators.ip_address.ipv4(ip):
            return '未知'
        reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-City.mmdb')
        ip_obj = reader.city(ip)
        return ip_obj.location.time_zone
    except Exception:
        return '未知'


def ip_get_continent(ip):
    """
    >>> 获取ip地址所在洲

    @params {String} ip:
    @returns: {String} ip所属洲
    """
    try:
        # 验证 IP 有效性
        if not validators.ip_address.ipv4(ip):
            return '未知'
        reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-City.mmdb')
        ip_obj = reader.city(ip)
        ip_continent_names = ip_obj.continent.names
        if ip_continent_names.get('zh-CN', None):
            return ip_continent_names['zh-CN']
        return ip_continent_names['en']
    except Exception:
        return '未知'


def ip_get_location(ip):
    """
    >>> 获取ip地址 经纬度

    @params {String} ip:
    @returns: {String} ip所属 经纬度
    """
    try:
        # 验证 IP 有效性
        if not validators.ip_address.ipv4(ip):
            return '未知'
        reader = geoip2.database.Reader('/usr/share/GeoIP/GeoLite2-City.mmdb')
        ip_obj = reader.city(ip)
        return [ip_obj.location.longitude, ip_obj.location.latitude]
    except Exception:
        return '未知'



# -*- coding: utf-8 -*-
"""
jf-ext.GeoExt.py
~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from math import radians, cos, sin, asin, sqrt


def geo_distance(lng1, lat1, lng2, lat2):
    """
    >>> 经纬度距离计算
    #公式计算两点间距离(km)
    :param {Float} lng1: 坐标1 经度
    :param {Float} lat1: 坐标1 维度
    :param {Float} lng2: 坐标2 经度
    :param {Float} lat2: 坐标2 维度
    """
    # 经纬度转换成弧度
    lng1, lat1, lng2, lat2 = map(radians, [float(lng1), float(lat1), float(lng2), float(lat2)])
    dlon = lng2 - lng1
    dlat = lat2 - lat1
    a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
    distance = 2 * asin(sqrt(a)) * 6371 * 1000   # 地球平均半径, 6371km
    distance = round(distance / 1000, 3)
    return distance



# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.ListExt.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""


def list_to_string(obj):
    """
    >>> list -> string
    :param {List} obj:
    :return {String}:
    """
    if isinstance(obj, list):
        return str(obj)
    return obj



# -*- coding: utf-8 -*-
"""
jf-ext.BasicType.FloatExt.py
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2010-2018 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""


def float_to_precision(x):
    """
    >>> 指定float精度
    :param {Float} x:
    :returns {Float}
    """
    if isinstance(x, float) or isinstance(x, int):
        return float('%0.2f' % x)
    else:
        return None



# -*- coding: utf-8 -*-
"""
jfExt.OrderExt.py
~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import random
from .Time.TimeExt import *


def order_gen_order_reference(order_type, date_string=None):
    """
    >>> order类: 生成订单 reference
    :param {String} order_type: 订单类型
    :param {String} date_string: 时间字符串 (default: None, 用当前时间)
    """
    if not date_string:
        date_string = time_now_string()
    date_string = date_string.replace("-", "", 10)
    date_string = date_string.replace(" ", "-", 10)
    date_string = date_string.replace(":", "", 10)
    random_id = random.randint(100, 999)
    order_reference = "{}-{}{}".format(order_type, date_string, random_id)
    return order_reference



# -*- coding: utf-8 -*-
"""
jf-ext.CommonExt.py
~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

import time


def dict_filter(dic, keys):
    new_dic = {}
    for key in keys:
        new_dic[key] = dic.get(key, "")
    return new_dic


def get_latency_str_for_millisecond(ms):
    if ms < 2:
        return "<2ms"
    elif ms < 5:
        return "2ms-5ms"
    elif ms < 20:
        return "5ms-20ms"
    elif ms < 100:
        return "20ms-100ms"
    elif ms < 1000:
        return "100ms-1s"
    elif ms < 5000:
        return "1s-5s"
    else:
        return ">5s"


def get_latency_msg_for_millisecond(ms, func_name):
    return ("[%4sms] [%10s] [%s]" % (ms, get_latency_str_for_millisecond(ms), func_name))


def timeout_tracking(func):
    """ 函数时间控制 """
    def wrapper(*argv, **kwgs):
        start_time = time.time()
        res = func(*argv, **kwgs)
        proc_time = int((time.time() - start_time) * 1000)
        msg = get_latency_msg_for_millisecond(proc_time, func.__name__)
        print(msg)
        return res
    return wrapper



# -*- coding: utf-8 -*-
"""
jfExt.ImageExt.py
~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from icecream import ic # noqa


def image_to_square(in_file, out_file, size=1024, background_color=(255, 255, 255)):
    """
    >>> 图片转换成正方形
    :param {String} in_file: 图片读取地址
    :param {String} out_file: 图片输出地址
    :param {Int} size: 图片长度/宽度
    :param {(Int, Int, Int)} background_color: 背景颜色
    """
    from PIL import Image
    image = Image.open(in_file)
    image = image.convert('RGB')
    w, h = image.size
    # 创建背景图，颜色值为127
    background = Image.new('RGB', size=(max(w, h), max(w, h)), color=(255, 255, 255))
    # 一侧需要填充的长度
    length = int(abs(w - h) // 2)
    # 粘贴的位置
    box = (length, 0) if w < h else (0, length)
    background.paste(image, box)
    # 缩放
    image_data = background.resize((1024, 1024))
    # background.show()
    image_data.save(out_file)



# -*- coding: utf-8 -*-
"""
msab.common.redisMgr.py
v1.0
~~~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2022 by the Ji Fu, see AUTHORS for more details.
:license: MIT, see LICENSE for more details.
"""

from flask_redis import FlaskRedis

from ..SingletonExt import Singleton


@Singleton
class RedisMgr():
    """
    >>> Redis Manager
    """

    def __init__(self):
        self._client = None

    @property
    def client(self):
        assert(self._client)    # ❗️❗️未初始化 Redis Client
        return self._client

    @client.setter
    def client(self, client):
        if isinstance(client, FlaskRedis):
            self._client = client

    def set_value(self, key, value, expire_time=864000):
        """
        >>> 设置key value
        :param {String} key: 键名称
        :param {String} value: 键值
        :param {Int} expire_time: 过期时间 (default 60s * 60m * 24h * 10 d)
        :returns {}:
        """
        if isinstance(key, str):
            self.client.set(key, value)
            self.client.expire(key, expire_time)
            return self.client.get(key)
        return None

    def get_value(self, key):
        """
        >>> 获取key value
        :param {String} key: 键名称
        :returns {String}: 键值
        """
        return self.client.get(key)

    def get_value_by_prefix(self, key_prefix):
        """
        >>> 通配符获取 Key Value
        :param {String} key_prefix: 键名 通配符
        :returns {List<String>}: list<键名key>
        """
        return self.client.keys(pattern="{}.*".format(key_prefix))

    def insert_set(self, key, value, expire_time=8640000):
        """
        >>> 插入 set 值
        :param {String} key: set键名称
        :param {String} value: 键值
        :param {Int} expire_time: 过期时间 (default 60s * 60m * 24h * 10 d)
        :returns {Boolean}: 插入结果
        """
        if isinstance(key, str):
            self.client.sadd(key, value)
            self.client.expire(key, expire_time)
            return True
        return None

    def get_set(self, key):
        """
        >>> 获取 set 内容
        :param {String} key: set键名称
        :param {String} value: 键值
        :returns {List<String>}: set 值
        """
        return self.client.smembers(key)

    def insert_list(self, key, value, expire_time=864000):
        """
        >>> 插入队列
        :param {String} key: 键名称
        :param {String} value: 键值
        :param {Int} expire_time: 过期时间 (default 60s * 60m * 24h * 10 d)
        :returns {list<>}: 插入后队列内容
        """
        if isinstance(key, str):
            self.client.lpush(key, value)
            self.client.expire(key, expire_time)
            return self.get_list(key)
        return None

    def pop_list(self, key):
        """
        >>> 出队
        :param {String} key: set键名称
        :returns {List<String>}: set
        """
        return self.client.lpop(key)

    def get_list(self, key):
        """
        >>> 获取队列内容
        :param {String} key: set键名称
        :returns {List<String>}: set
        """
        llen = self.client.llen(key)
        return self.client.lrange(key, 0, llen)

    def remaining(self, key):
        """
        >>> 查询过期时间
        :param {String} key: 键名称
        :param {Int}: 过期时间
        """
        return self.client.ttl(key)

    def rm_key(self, key):
        """
        >>> 删除键
        :param {String} key: 键名称
        :returns {Boolean}: 插入结果
        """
        return self.client.delete(key)

    def rm_keys(self, key_prefix):
        """
        >>> 删除一组键值对
        :param {String} key_prefix: 键名称 prefix
        :returns {Boolean}: 插入结果
        """
        keys = self.client.keys("{}*".format(key_prefix))
        for key in keys:
            self.client.delete(key)

    def flush_db(self):
        """
        >>> 清空应用 cache
        """
        cache_keys = self.keys("msab.cache*")
        for i in cache_keys:
            self.client.delete(i)
